package io.zipcoder.tc_spring_poll_application.controller;

import io.zipcoder.tc_spring_poll_application.domain.Option;
import io.zipcoder.tc_spring_poll_application.domain.Poll;
import io.zipcoder.tc_spring_poll_application.domain.Vote;
import io.zipcoder.tc_spring_poll_application.dtos.OptionCount;
import io.zipcoder.tc_spring_poll_application.dtos.VoteResult;
import io.zipcoder.tc_spring_poll_application.repositories.VoteRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;

@RestController
public class ComputeResultController {

    private VoteRepository voteRepository;

    @Autowired
    public ComputeResultController(VoteRepository voteRepository) {
        this.voteRepository = voteRepository;
    }

    @RequestMapping(value = "/computeresult", method = RequestMethod.GET)
    public ResponseEntity<?> computeResult(@RequestParam Long pollId) {
        VoteResult voteResult = new VoteResult();
        Map<Long, Integer> mapOptions = mapVotes(voteRepository.findById(pollId));
        Collection<OptionCount> optionCounts = collectOptionCounts(mapOptions);

        voteResult.setResults(optionCounts);
        voteResult.setTotalVotes(getTotalVote(optionCounts));

        return new ResponseEntity<>(voteResult, HttpStatus.OK);
    }

    private Map<Long, Integer> mapVotes(Iterable<Vote> allVotes) {
        Map<Long, Integer> mapOptions = new HashMap<>();
        for(Vote vote: allVotes) {
            Long id = vote.getOption().getId();
            if(!mapOptions.containsKey(id)) {
                mapOptions.put(id, 1);
            } else {
                int count = mapOptions.get(id);
                count = count + 1;
                mapOptions.put(id, count);
            }
        }
        return mapOptions;
    }

    private Collection<OptionCount> collectOptionCounts (Map<Long, Integer> mapOptions) {
        Collection<OptionCount> optionCounts = new ArrayList<>();

        for(Long i: mapOptions.keySet()) {
            OptionCount optionCount = new OptionCount(i, mapOptions.get(i));
            optionCounts.add(optionCount);
        }
        return optionCounts;
    }

    private int getTotalVote(Collection<OptionCount> optionCounts) {
        int totalVotes = 0;

        for(OptionCount o: optionCounts) {
            totalVotes += o.getCount();
        }
        return totalVotes;
    }
}